/*
 * wireless_lightstrap.h
 *
 * wireless lightstrap driver
 *
 * Copyright (c) 2020-2020 Huawei Technologies Co., Ltd.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 */

#ifndef _WIRELESS_LIGHTSTRAP_H_
#define _WIRELESS_LIGHTSTRAP_H_

#include <linux/device.h>
#include <linux/notifier.h>
#include <linux/workqueue.h>
#include <huawei_platform/power/wireless/wireless_tx_pwr_src.h>
#include <huawei_platform/power/wireless/wireless_transmitter.h>

struct lightstrap_di {
	struct device *dev;
	struct notifier_block event_nb;
	struct work_struct event_work;
	struct delayed_work check_work;
	u8 product_type;
	unsigned long event_type;
	bool is_opened_by_hall;
	unsigned int ping_freq;
	unsigned int work_freq;
};

enum wireless_lightstrap_sysfs_type {
	LIGHTSTRAP_SYSFS_DEV_PRODUCT_TYPE,
};

#ifdef CONFIG_WIRELESS_ACCESSORY
bool lightstrap_online_state(void);
enum wltx_pwr_src lightstrap_specify_pwr_src(void);
void lightstrap_reinit_tx_chip(void);
#else
static inline bool lightstrap_online_state(void)
{
	return false;
}

static inline enum wltx_pwr_src lightstrap_specify_pwr_src(void)
{
	return PWR_SRC_NULL;
}

static inline void lightstrap_reinit_tx_chip(void) {}
#endif /* CONFIG_WIRELESS_ACCESSORY */

#endif /* _WIRELESS_LIGHTSTRAP_H_ */
