/*
 * hisi_rtg.h
 *
 * related thread grop.
 *
 * Copyright (c) 2018-2020 Huawei Technologies Co., Ltd.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 */
#ifndef __HISI_RTG_H
#define __HISI_RTG_H

#include <linux/hisi/perf_ctrl.h>

struct group_cpu_time {
	u64 window_start;
	u64 curr_runnable_sum;
	u64 prev_runnable_sum;
};

struct group_time {
	unsigned long curr_window_load;
	unsigned long curr_window_exec;
	unsigned long prev_window_load;
	unsigned long prev_window_exec;
	unsigned long normalized_util;
};

enum RTG_GRP_ID {
	DEFAULT_RTG_GRP_ID,
	DEFAULT_CGROUP_COLOC_ID = 1,
	DEFAULT_AI_ID = 2,
	DEFAULT_AI_RENDER_THREAD_ID = 3,
	DEFAULT_AI_OTHER_THREAD_ID = 4,
	DEFAULT_RT_FRAME_ID = 8,
	DEFAULT_AUX_ID = 9,
	MAX_NUM_CGROUP_COLOC_ID = 10,
};

struct grp_load_mode {
	bool freq_enabled;
	bool util_enabled;
};

struct rtg_class;

struct related_thread_group {
	int id;
	raw_spinlock_t lock;
	struct list_head tasks;
	struct list_head list;
	struct group_cpu_time * __percpu cpu_time;
	struct sched_cluster *preferred_cluster;
	struct group_time time;
	struct group_time time_pref_cluster;
	unsigned long freq_update_interval; /* in nanoseconds */
	unsigned long util_invalid_interval; /* in nanoseconds */
	unsigned long util_update_timeout; /*in nanoseconds */
	u64 last_freq_update_time;
	u64 last_util_update_time;
	u64 window_start;
	u64 mark_start;
	u64 prev_window_time;
	/* rtg window information for WALT */
	unsigned int window_size;
	unsigned int nr_running;
	/* the min freq set by userspace */
	unsigned int us_set_min_freq;
	int max_boost;
	struct grp_load_mode mode;
#ifdef CONFIG_HISI_ED_TASK
	bool ed_enabled;
	unsigned int ed_task_running_duration;
	unsigned int ed_task_waiting_duration;
	unsigned int ed_new_task_running_duration;
#endif /* CONFIG_HISI_ED_TASK */

	void *private_data;
	const struct rtg_class *rtg_class;
	unsigned int capacity_margin;
};

struct rtg_class {
	void (*sched_update_rtg_tick)(struct related_thread_group *grp);
};

enum rtg_freq_update_flags {
	FRAME_FORCE_UPDATE = (1 << 0),
	FRAME_NORMAL_UPDATE = (1 << 1),
	AI_FORCE_UPDATE = (1 << 2),
};

#ifdef CONFIG_HISI_RTG
int sched_set_group_id(pid_t pid, unsigned int group_id);
struct related_thread_group* lookup_related_thread_group(unsigned int group_id);
unsigned int get_cluster_grp_running(int cluster_id);
int sched_set_preferred_cluster(unsigned int grp_id, int sched_cluster_id);
int sched_set_group_normalized_util(unsigned int grp_id, unsigned long util,
				    unsigned int flag);
int sched_set_group_window_size(unsigned int grp_id, unsigned int rate);
int sched_set_group_window_rollover(unsigned int grp_id);
int sched_set_group_freq(unsigned int grp_id, unsigned int freq);
int sched_set_group_freq_update_interval(unsigned int grp_id,
					 unsigned int interval);
int sched_set_group_util_invalid_interval(unsigned int grp_id,
					  unsigned int interval);
int sched_set_group_load_mode(struct rtg_load_mode *mode);
int sched_set_group_ed_params(struct rtg_ed_params *params);
unsigned int sched_get_group_id(struct task_struct *p);

#else /* CONFIG_HISI_RTG */

static inline struct related_thread_group*
	lookup_related_thread_group(unsigned int group_id)
{
	return NULL;
}

static inline unsigned int get_cluster_grp_running(int cluster_id)
{
	return 0;
}

static inline int sched_set_group_id(pid_t pid, unsigned int group_id)
{
	return 0;
}

static inline int sched_set_preferred_cluster(unsigned int grp_id,
					      int sched_cluster_id)
{
	return 0;
}

static inline int sched_set_group_normalized_util(unsigned int grp_id,
						  unsigned long util,
						  unsigned int flag)
{
	return 0;
}

static inline int sched_set_group_window_size(unsigned int grp_id,
					      unsigned int rate)
{
	return 0;
}

static inline int sched_set_group_window_rollover(unsigned int grp_id)
{
	return 0;
}

static inline int sched_set_group_freq(unsigned int grp_id, unsigned int freq)
{
	return 0;
}

static inline int sched_set_group_freq_update_interval(unsigned int grp_id,
						       unsigned int interval)
{
	return 0;
}

static inline int sched_set_group_util_invalid_interval(unsigned int grp_id,
							unsigned int interval)
{
	return 0;
}

static inline int sched_set_group_load_mode(struct rtg_load_mode *mode)
{
	return 0;
}

static inline int sched_set_group_ed_params(struct rtg_ed_params *params)
{
	return 0;
}

static inline unsigned int sched_get_group_id(struct task_struct *p)
{
	return 0;
}

#endif

#endif
